﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Text.RegularExpressions;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 棋譜
    /// </summary>
    public class Kifu
    {

        #region プロパティー類
        //------------------------------------------------------------
        private string startpos;

        /// <summary>
        /// 初期配置。
        /// </summary>
        public string Startpos
        {
            get
            {
                return this.startpos;
            }
        }
        //------------------------------------------------------------
        private List<Process> processList;

        /// <summary>
        /// 符号リスト
        /// </summary>
        public List<Process> ProcessList
        {
            get
            {
                return this.processList;
            }
        }
        //------------------------------------------------------------
        private Kyokumen kyokumen;

        public Kyokumen Kyokumen
        {
            get
            {
                return this.kyokumen;
            }
        }
        //------------------------------------------------------------
        #endregion




        public Kifu()
        {

            this.kyokumen = new Kyokumen();
            this.startpos = "未設定";
            this.processList = new List<Process>();
        }

        public Kifu Clone()
        {
            Kifu kifu2 = new Kifu();

            //------------------------------------------------------------
            // コピー
            //------------------------------------------------------------
            kifu2.startpos = this.startpos;

            foreach (Process masuBangoFugo in this.processList)
            {
                kifu2.processList.Add(masuBangoFugo);
            }

            kifu2.kyokumen = this.Kyokumen.Clone();

            return kifu2;
        }

        public void SetStartpos()
        {
            this.startpos = "startpos";
        }

        public void SetStartpos(string startpos)
        {
            this.startpos = startpos;
        }

        /// <summary>
        /// ここから採譜
        /// </summary>
        public void SetStartpos_KokokaraSaifu(Sengo sengo)
        {
            StringBuilder sb = new StringBuilder();

            for (int dan = 1; dan <= 9; dan++)
            {
                int spaceCount = 0;

                for (int suji = 9; suji >= 1; suji--)
                {
                    Button_Koma btn = this.Kyokumen.Btn_AtMasuDokodemo(Okiba.SHOGIBAN, new Process(
                        Sengo.NO_USE_ARG,
                        Okiba.SHOGIBAN,
                        suji,
                        dan,
                        KomaSyurui.UNKNOWN
                        ));

                    if (null != btn)
                    {
                        if (0 < spaceCount)
                        {
                            sb.Append(spaceCount);
                            spaceCount = 0;
                        }
                        sb.Append(Converter.SyuruiToSfen(btn.KomaProperty.Sengo, btn.KomaProperty.Syurui));
                    }
                    else
                    {
                        spaceCount++;
                    }

                }

                if (0 < spaceCount)
                {
                    sb.Append(spaceCount);
                    spaceCount = 0;
                }

                if (dan != 9)
                {
                    sb.Append("/");
                }
            }

            sb.Append(" ");

            //------------------------------------------------------------
            // 先後
            //------------------------------------------------------------
            switch (sengo)
            {
                case Sengo.GOTE:
                    sb.Append("w");
                    break;
                default:
                    sb.Append("b");
                    break;
            }

            sb.Append(" ");

            //------------------------------------------------------------
            // 持ち駒
            //------------------------------------------------------------
            {
                int mK = 0;
                int mR = 0;
                int mB = 0;
                int mG = 0;
                int mS = 0;
                int mN = 0;
                int mL = 0;
                int mP = 0;

                int mk = 0;
                int mr = 0;
                int mb = 0;
                int mg = 0;
                int ms = 0;
                int mn = 0;
                int ml = 0;
                int mp = 0;

                // 先手
                List<Button_Koma> komasS = this.Kyokumen.Btns_ByOkiba(Okiba.SENTE_KOMADAI);
                foreach (Button_Koma btn in komasS)
                {
                    switch (Converter.SyuruiToFunariCase(btn.KomaProperty.Syurui))
                    {
                        case KomaSyurui.OH:
                            mK++;
                            break;
                        case KomaSyurui.HISYA:
                            mR++;
                            break;
                        case KomaSyurui.KAKU:
                            mB++;
                            break;
                        case KomaSyurui.KIN:
                            mG++;
                            break;
                        case KomaSyurui.GIN:
                            mS++;
                            break;
                        case KomaSyurui.KEI:
                            mN++;
                            break;
                        case KomaSyurui.KYO:
                            mL++;
                            break;
                        case KomaSyurui.FU:
                            mP++;
                            break;
                    }
                }

                // 後手
                List<Button_Koma> komasG = this.Kyokumen.Btns_ByOkiba(Okiba.GOTE_KOMADAI);
                foreach (Button_Koma btn in komasG)
                {
                    switch (Converter.SyuruiToFunariCase(btn.KomaProperty.Syurui))
                    {
                        case KomaSyurui.OH:
                            mk++;
                            break;
                        case KomaSyurui.HISYA:
                            mr++;
                            break;
                        case KomaSyurui.KAKU:
                            mb++;
                            break;
                        case KomaSyurui.KIN:
                            mg++;
                            break;
                        case KomaSyurui.GIN:
                            ms++;
                            break;
                        case KomaSyurui.KEI:
                            mn++;
                            break;
                        case KomaSyurui.KYO:
                            ml++;
                            break;
                        case KomaSyurui.FU:
                            mp++;
                            break;
                    }
                }

                if (0 == mK + mR + mB + mG + mS + mN + mL + mP + mk + mr + mb + mg + ms + mn + ml + mp)
                {
                    sb.Append("-");
                }
                else
                {
                    if (0 < mK)
                    {
                        sb.Append("K");
                        sb.Append(mK);
                    }

                    if (0 < mR)
                    {
                        sb.Append("R");
                        sb.Append(mR);
                    }

                    if (0 < mB)
                    {
                        sb.Append("B");
                        sb.Append(mB);
                    }

                    if (0 < mG)
                    {
                        sb.Append("G");
                        sb.Append(mG);
                    }

                    if (0 < mS)
                    {
                        sb.Append("S");
                        sb.Append(mS);
                    }

                    if (0 < mN)
                    {
                        sb.Append("N");
                        sb.Append(mN);
                    }

                    if (0 < mL)
                    {
                        sb.Append("L");
                        sb.Append(mL);
                    }

                    if (0 < mP)
                    {
                        sb.Append("P");
                        sb.Append(mP);
                    }

                    if (0 < mk)
                    {
                        sb.Append("k");
                        sb.Append(mk);
                    }

                    if (0 < mr)
                    {
                        sb.Append("r");
                        sb.Append(mr);
                    }

                    if (0 < mb)
                    {
                        sb.Append("b");
                        sb.Append(mb);
                    }

                    if (0 < mg)
                    {
                        sb.Append("g");
                        sb.Append(mg);
                    }

                    if (0 < ms)
                    {
                        sb.Append("s");
                        sb.Append(ms);
                    }

                    if (0 < mn)
                    {
                        sb.Append("n");
                        sb.Append(mn);
                    }

                    if (0 < ml)
                    {
                        sb.Append("l");
                        sb.Append(ml);
                    }

                    if (0 < mp)
                    {
                        sb.Append("p");
                        sb.Append(mp);
                    }
                }

            }

            // 手目
            sb.Append(" 1");

            //------------------------------------------------------------
            // 棋譜を空に
            //------------------------------------------------------------
            this.processList.Clear();

            this.SetStartpos(sb.ToString());
        }

        /// <summary>
        /// 手目。
        /// </summary>
        public int LastTeme
        {
            get
            {
                return this.processList.Count;
            }
        }

        private int LastIndex
        {
            get
            {
                return this.processList.Count - 1;
            }
        }

        public Process Last
        {
            get
            {
                Process last = null;

                if (0 != this.processList.Count)
                {
                    last = this.processList.Last();
                }

                return last;
            }
        }

        /// <summary>
        /// 棋譜に符号を追加。
        /// 
        /// ①コマ送り用。
        /// ②「成り」フラグの更新用。
        /// ③マウス操作用
        /// 
        /// </summary>
        /// <param name="process"></param>
        public void Add(Process process)
        {
            this.processList.Add(process);
        }


        /// <summary>
        /// 最後の要素を切り取って返します。なければヌル。
        /// </summary>
        /// <returns></returns>
        public void PopLast(out Process last)
        {
            last = this.Last;

            if (null != last)
            {
                this.processList.RemoveAt(this.processList.Count - 1);
            }
        }

        public void ElementAt(int index, out Process process)
        {
            process = null;

            if(0<=index && index<this.processList.Count)
            {
                process = this.processList[index];
            }
        }


        public void Clear()
        {
            this.processList.Clear();
        }

    }


}
