﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 対局。
    /// </summary>
    public class Taikyoku
    {
        #region プロパティー類
        //------------------------------------------------------------
        /// <summary>
        /// 将棋盤
        /// </summary>
        public Shogiban Shogiban
        {
            get;
            set;
        }
        //------------------------------------------------------------
        /// <summary>
        /// 駒置き。[0]先手、[1]後手。
        /// </summary>
        public Komadai[] KomadaiArr
        {
            get;
            set;
        }
        //------------------------------------------------------------
        private Kifu kifu;

        /// <summary>
        /// 棋譜
        /// </summary>
        public Kifu Kifu
        {
            get
            {
                return this.kifu;
            }
        }
        //------------------------------------------------------------
        private Sengo sengo;

        /// <summary>
        /// 現在の先後
        /// </summary>
        public Sengo Sengo
        {
            get
            {
                return this.sengo;
            }
        }

        public void SetSengo(Sengo sengo)
        {
            this.sengo = sengo;
        }
        //------------------------------------------------------------
        private int ittemaeSuji;

        /// <summary>
        /// 一手前の筋
        /// </summary>
        public int IttemaeSuji
        {
            get
            {
                return this.ittemaeSuji;
            }
        }

        private int ittemaeDan;

        /// <summary>
        /// 一手前の段
        /// </summary>
        public int IttemaeDan
        {
            get
            {
                return this.ittemaeDan;
            }
        }
        //------------------------------------------------------------
        private Process mouseProcessOrNull;

        /// <summary>
        /// 棋譜再生中は使いません。
        /// 
        /// マウスで駒を動かしたときに使います。
        /// </summary>
        public Process MouseProcessOrNull
        {
            get
            {
                return this.mouseProcessOrNull;
            }
        }

        public void SetMouseProcessOrNull(Process mouse)
        {
            this.mouseProcessOrNull = mouse;
        }
        //------------------------------------------------------------
        #endregion


        
        //------------------------------------------------------------

        public Taikyoku()
        {
            this.kifu = new Kifu();

            //----------
            // 将棋盤
            //----------
            this.Shogiban = new Shogiban(200, 220);

            //----------
            // 駒置き
            //----------
            this.KomadaiArr = new Komadai[2];
            this.KomadaiArr[0] = new Komadai(Okiba.SENTE_KOMADAI, 610, 220, Sengo.SENTE);
            this.KomadaiArr[1] = new Komadai(Okiba.GOTE_KOMADAI, 10, 170, Sengo.GOTE);

            this.Init();
        }

        public void Init()
        {
            this.ittemaeSuji = Process.INITIALIZED_SUJI;
            this.ittemaeDan = Process.INITIALIZED_DAN;
            this.sengo = Sengo.SENTE;
        }

        public void ChangeTurn(int lastSuji, int lastDan, Uc_Main uc_Main)
        {
            this.ittemaeSuji = lastSuji;
            this.ittemaeDan = lastDan;
            //System.Console.WriteLine("一手前筋：" + this.ittemaeSuji + "　一手前段：" + this.ittemaeDan);

            this.sengo = Converter.AlternateSengo(this.sengo);

            uc_Main.LblSengo.Text = Converter.SengoToKanji(uc_Main.Taikyoku.Sengo);
        }


        /// <summary>
        /// 駒を駒袋に移動させます。
        /// 
        /// 駒袋には、不成にして入れておきます。
        /// </summary>
        public void ClearToKomabukuro()
        {
            System.Console.WriteLine("全駒 this.Kifu.Kyokumen.Items.Count=" + this.Kifu.Kyokumen.Items.Count);


            foreach (Button_Koma koma in this.Kifu.Kyokumen.Items)
            {
                koma.KomaProperty = new Process(
                    koma.KomaProperty.Sengo,
                    Okiba.KOMABUKURO,
                    Process.TUKAWANAI_SRCSRC_SUJI,
                    Process.TUKAWANAI_SRCSRC_DAN,
                    Converter.SyuruiToFunariCase( koma.KomaProperty.Syurui)
                    );

                System.Console.WriteLine("　　koma.Masu.ToText()=" + koma.KomaProperty.ToText());
            }
        }

    }
}
