﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 駒の操作
    /// </summary>
    public abstract class Actions_Koma
    {

        /// <summary>
        /// 将棋盤オブジェクトを利用して、駒のx,y位置を変更します
        /// </summary>
        /// <param name="koma">駒</param>
        public static void RefreshKomaLocation( Button_Koma koma, Taikyoku taikyoku )
        {
            switch (koma.KomaProperty.Okiba)
            {
                case Okiba.SHOGIBAN:
                    koma.Bounds = new Rectangle(
                        taikyoku.Shogiban.SujiToX(koma.KomaProperty.Suji),
                        taikyoku.Shogiban.DanToY(koma.KomaProperty.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;

                case Okiba.SENTE_KOMADAI:
                    koma.Bounds = new Rectangle(
                        taikyoku.KomadaiArr[0].SujiToX(koma.KomaProperty.Suji),
                        taikyoku.KomadaiArr[0].DanToY(koma.KomaProperty.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;

                case Okiba.GOTE_KOMADAI:
                    koma.Bounds = new Rectangle(
                        taikyoku.KomadaiArr[1].SujiToX(koma.KomaProperty.Suji),
                        taikyoku.KomadaiArr[1].DanToY(koma.KomaProperty.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;
            }
        }


        /// <summary>
        /// 駒台の空いている場所を返します。無ければヌル。
        /// </summary>
        /// <param name="okiba">置き場</param>
        /// <param name="uc_Main">メインパネル</param>
        /// <returns>置ける場所</returns>
        public static Process GetKomadaiSpace( Okiba okiba, Taikyoku taikyoku )
        {
            Process masu = null;

            int komadaiIndex = 0;
            if (okiba == Okiba.GOTE_KOMADAI)
            {
                komadaiIndex = 1;
            }


            bool[,] exists = new bool[taikyoku.KomadaiArr[komadaiIndex].LastDan, taikyoku.KomadaiArr[komadaiIndex].LastSuji];

            foreach (Button_Koma k in taikyoku.Kifu.Kyokumen.Items)
            {
                if (k.KomaProperty.Okiba == okiba)
                {
                    exists[k.KomaProperty.Dan - 1, k.KomaProperty.Suji - 1] = true;
                }
            }

            Sengo sengo;
            switch (okiba)
            {
                case Okiba.GOTE_KOMADAI:
                    sengo = Sengo.GOTE;
                    break;
                default:
                    sengo = Sengo.SENTE;
                    break;
            }

            for (int dan = 1; dan <= taikyoku.KomadaiArr[komadaiIndex].LastDan; dan++)
            {
                for (int suji = 1; suji <= taikyoku.KomadaiArr[komadaiIndex].LastSuji; suji++)
                {
                    if (!exists[dan-1, suji-1])
                    {
                        masu = new Process(
                            sengo,
                            okiba,
                            suji,
                            dan,
                            KomaSyurui.UNKNOWN
                            );
                        goto gt_EndMethod;
                    }
                }
            }

        gt_EndMethod:
            return masu;
        }

    }

}
