﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;



namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 駒の軌道。
    /// </summary>
    public abstract class KomanoKidou
    {


        public static List<Process> SrcA_Hisya(Sengo sengo, Process itaru)
        {
            List<Process> srcA = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ａ

                for (int i = 0; i < 8; i++) { srcA.Add( itaru.Offset(sengo, 0, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　                  Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcA.Add( itaru.Offset(sengo, 0, -(i + 1))); }
            }

            return srcA;
        }


        public static List<Process> SrcC_Hisya(Sengo sengo, Process itaru)
        {
            List<Process> srcC = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C7│C6│C5│C4│C3│C2│C1│C0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcC.Add( itaru.Offset(sengo, i + 1, 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcC.Add( itaru.Offset(sengo, -(i + 1), 0)); }
            }

            return srcC;
        }


        public static List<Process> SrcE_Hisya(Sengo sengo, Process itaru)
        {
            List<Process> srcE = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　                  Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcE.Add( itaru.Offset(sengo, 0, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ｅ

                for (int i = 0; i < 8; i++) { srcE.Add( itaru.Offset(sengo, 0, i + 1)); }
            }

            return srcE;
        }


        public static List<Process> SrcG_Hisya(Sengo sengo, Process itaru)
        {
            List<Process> srcG = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcG.Add( itaru.Offset(sengo, -(i + 1), 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G7│G6│G5│G4│G3│G2│G1│G0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcG.Add( itaru.Offset(sengo, i + 1, 0)); }
            }

            return srcG;
        }


        public static List<Process> SrcB_Kaku(Sengo sengo, Process itaru)
        {
            List<Process> srcB = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B7│  │  │  │  │  │  │  │  │
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcB.Add( itaru.Offset(sengo, i + 1, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│  │  │  │  │  │  │  │  │B7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcB.Add( itaru.Offset(sengo, -(i + 1), -(i + 1))); }
            }

            return srcB;
        }


        public static List<Process> SrcD_Kaku(Sengo sengo, Process itaru)
        {
            List<Process> srcD = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│D7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcD.Add( itaru.Offset(sengo, i + 1, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │D7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                for (int i = 0; i < 8; i++) { srcD.Add( itaru.Offset(sengo, -(i + 1), i + 1)); }
            }

            return srcD;
        }


        public static List<Process> SrcF_Kaku(Sengo sengo, Process itaru)
        {
            List<Process> srcF = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│  │  │  │  │  │  │  │  │F7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcF.Add( itaru.Offset(sengo, -(i + 1), -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F7│  │  │  │  │  │  │  │  │
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcF.Add( itaru.Offset(sengo, i + 1, i + 1)); }
            }

            return srcF;
        }


        public static List<Process> SrcH_Kaku(Sengo sengo, Process itaru)
        {
            List<Process> srcH = new List<Process>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │H7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                for (int i = 0; i < 8; i++) { srcH.Add( itaru.Offset(sengo, -(i + 1), i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│H7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcH.Add( itaru.Offset(sengo, i + 1, -(i + 1))); }
            }

            return srcH;
        }


        public static List<Process> SrcA_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcA;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │Ａ│  │
                //└─┴─┴─┘

                srcA = new List<Process>() { itaru.Offset(sengo, 0, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │Ａ│  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┴─┴─┘

                srcA = new List<Process>() { itaru.Offset(sengo, 0, -1) };
            }

            return srcA;
        }


        public static List<Process> SrcB_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcB;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│Ｂ│  │  │
                //└─┴─┴─┘

                srcB = new List<Process>() { itaru.Offset(sengo, 1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │Ｂ│
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcB = new List<Process>() { itaru.Offset(sengo, -1, -1) };
            }

            return srcB;
        }


        public static List<Process> SrcC_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcC;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│Ｃ│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcC = new List<Process>() { itaru.Offset(sengo, 1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│　│至│Ｃ│
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcC = new List<Process>() { itaru.Offset(sengo, -1, 0) };
            }

            return srcC;
        }


        public static List<Process> SrcD_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcD;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│Ｄ│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcD = new List<Process>() { itaru.Offset(sengo, 1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │Ｄ│
                //└─┴─┴─┘

                srcD = new List<Process>() { itaru.Offset(sengo, -1, 1) };
            }

            return srcD;
        }


        public static List<Process> SrcE_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcE;


            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│　│Ｅ│  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcE = new List<Process>() { itaru.Offset(sengo, 0, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│Ｅ│  │
                //└─┴─┴─┘

                srcE = new List<Process>() { itaru.Offset(sengo, 0, 1) };
            }


            return srcE;
        }


        public static List<Process> SrcF_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcF;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │Ｆ│
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcF = new List<Process>() { itaru.Offset(sengo, -1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │Ｆ│  │  │
                //  └─┴─┴─┘

                srcF = new List<Process>() { itaru.Offset(sengo, 1, 1) };
            }

            return srcF;
        }


        public static List<Process> SrcG_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcG;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│Ｇ│
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcG = new List<Process>() { itaru.Offset(sengo, -1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │Ｇ│至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcG = new List<Process>() { itaru.Offset(sengo, 1, 0) };
            }

            return srcG;
        }


        public static List<Process> SrcH_Oh(Sengo sengo, Process itaru)
        {
            List<Process> srcH;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │Ｈ│
                //  └─┴─┴─┘

                srcH = new List<Process>() { itaru.Offset(sengo, -1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │Ｈ│  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcH = new List<Process>() { itaru.Offset(sengo, 1, -1) };
            }

            return srcH;
        }


        public static List<Process> SrcI_Kei(Sengo sengo, Process itaru)
        {
            List<Process> srcI;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│Ｉ│　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┘　└─┘

                srcI = new List<Process>() { itaru.Offset(sengo, 1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │　│Ｉ│
                //└─┘　└─┘

                srcI = new List<Process>() { itaru.Offset(sengo, -1, 2) };
            }

            return srcI;
        }


        public static List<Process> SrcJ_Kei(Sengo sengo, Process itaru)
        {
            List<Process> srcJ;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│  │　│Ｊ│
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┘　└─┘

                //----------
                // 競合駒マス(pre masu)
                //----------
                srcJ = new List<Process>() { itaru.Offset(sengo, -1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│Ｊ│　│  │
                //└─┘　└─┘

                srcJ = new List<Process>() { itaru.Offset(sengo, 1, 2) };
            }

            return srcJ;
        }

    }

}
