﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;
using System.Text.RegularExpressions;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 「▲７六歩」といった記述を、棋譜データの持ち方に変換します。
    /// </summary>
    public abstract class Translator_FugotextToProcess
    {

        /// <summary>
        /// テキスト形式の符号「▲７六歩△３四歩▲２六歩…」の最初の要素を、切り取ってプロセスに変換します。
        /// 
        /// 再生、コマ送りで利用。
        /// </summary>
        /// <returns></returns>
        public static bool TranslateFugotextToProcess(string text, out string restText, out Process process, Kifu baseKifu, Taikyoku taikyoku)
        {
            process = null;

            bool successful = false;

            restText = text;

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            Regex regex = new Regex(
                @"^\s*([▲△]?)(?:([123456789１２３４５６７８９])([123456789１２３４５６７８９一二三四五六七八九]))?(同)?[\s　]*(歩|香|桂|銀|金|飛|角|王|玉|と|成香|成桂|成銀|竜|龍|馬)(右|左|直)?(寄|引|上)?(成|不成)?(打?)",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    Translator_FugotextToProcess.TranslateArgsToProcess(
                        m.Groups[1].Value,  //▲△
                        m.Groups[2].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[3].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[4].Value,  // “同”
                        m.Groups[5].Value,  //(歩|香|桂|…
                        m.Groups[6].Value,           // 右|左…
                        m.Groups[7].Value,  // 上|引
                        m.Groups[8].Value, //成|不成
                        m.Groups[9].Value,  //打
                        out process,
                        baseKifu.Kyokumen,
                        taikyoku
                        );
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


        /// <summary>
        /// 符号１「▲６８銀上」を元に、「7968」を作ります。
        /// 
        /// ＜再生、コマ送りで呼び出されます＞
        /// </summary>
        /// <returns></returns>
        public static void TranslateArgsToProcess(
            string strSengo, //▲△
            string strSuji, //123…9、１２３…９、一二三…九
            string strDan, //123…9、１２３…９、一二三…九
            string strDou, // “同”
            string strSrcSyurui, //(歩|香|桂|…
            string strMigiHidari,           // 右|左…
            string strAgaruHiku, // 上|引
            string strNariFunari, //成|不成
            string strDaHyoji, //打
            out Process process,
            Kyokumen kyokumen,
            Taikyoku taikyoku
            )
        {
            //------------------------------
            // 符号確定
            //------------------------------
            MigiHidari migiHidari = Converter.StrToMigiHidari(strMigiHidari);
            AgaruHiku agaruHiku = Converter.StrToAgaruHiku(strAgaruHiku);            // 上|引
            NariFunari nariFunari = Converter.NariToBool(strNariFunari);//成
            DaHyoji daHyoji = Converter.StrToDaHyoji(strDaHyoji);             //打

            KomaSyurui srcSyurui = Converter.MojiToSyurui(strSrcSyurui);


            //------------------------------
            // 
            //------------------------------
            Sengo sengo = Converter.SengoToEnum(strSengo);


            int suji;
            int dan;

            if ("同" == strDou)
            {
                // 1手前の筋、段を求めます。
                suji = taikyoku.IttemaeSuji;
                dan = taikyoku.IttemaeDan;
            }
            else
            {
                suji = Converter.SujiToInt(strSuji);
                dan = Converter.SujiToInt(strDan);
            }

            // TODO: 駒台にあるかもしれない。
            Okiba srcOkiba = Okiba.SHOGIBAN; //Okiba.FUMEI_FUGO_YOMI_CHOKUGO;// Okiba.SHOGIBAN;
            if (DaHyoji.Visible == daHyoji)
            {
                if (Sengo.GOTE == sengo)
                {
                    srcOkiba = Okiba.GOTE_KOMADAI;
                }
                else
                {
                    srcOkiba = Okiba.SENTE_KOMADAI;
                }
            }

            Process dst1 = new Process(
                sengo,
                Okiba.SHOGIBAN,
                //srcOkiba,
                suji,
                dan,
                //Process.ITTEMAE_KYOKUMEN_SUJI, // 符号からは、元位置は分かりません。
                //Process.ITTEMAE_KYOKUMEN_DAN, // 符号からは、元位置は分かりません。
                srcSyurui
                //KomaSyurui.UNKNOWN,
                //KomaSyurui.UNKNOWN//取った駒は分からない
                );

            Button_Koma found = null;

            //----------
            // 駒台の駒を(明示的に)打つなら
            //----------
            bool utsu = false;//駒台の駒を打つなら真
            if (DaHyoji.Visible == daHyoji)
            {
                utsu = true;
                goto gt_EndShogiban;
            }


            switch (srcSyurui)
            {
                #region 歩
                //************************************************************
                // 歩
                //************************************************************
                case KomaSyurui.FU:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│  │  │  │
                        //├─┼─┼─┤
                        //│  │至│  │
                        //├─┼─┼─┤
                        //│  │Ｅ│  │
                        //└─┴─┴─┘
                        bool isE = true;

                        List<Process> srcAll = new List<Process>();
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 飛
                //************************************************************
                // 飛
                //************************************************************
                case KomaSyurui.HISYA:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isC = true;
                        bool isE = true;
                        bool isG = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isE = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isE = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isG = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isC = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Hisya(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Hisya(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Hisya(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 角
                //************************************************************
                // 角
                //************************************************************
                case KomaSyurui.KAKU:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│  │B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │至│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│  │D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isB = true;
                        bool isD = true;
                        bool isF = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isB = false;
                                isD = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isB = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isB = false;
                                isH = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isF = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isB = false;
                                isD = false;
                                break;
                            case MigiHidari.Sugu:
                                isD = false;
                                isF = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Kaku(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Kaku(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Kaku(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Kaku(sengo, dst1)); }

                        //----------
                        // 候補マスＢ
                        //----------
                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 香
                //************************************************************
                // 香
                //************************************************************
                case KomaSyurui.KYO:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┐
                        //  │  │至│  │
                        //  ├─┼─┼─┤
                        //  │  │E0│  │
                        //  ├─┼─┼─┤
                        //  │  │E1│  │
                        //  ├─┼─┼─┤
                        //  │  │E2│  │
                        //  ├─┼─┼─┤
                        //  │  │E3│  │
                        //  ├─┼─┼─┤
                        //  │  │E4│  │
                        //  ├─┼─┼─┤
                        //  │  │E5│  │
                        //  ├─┼─┼─┤
                        //  │  │E6│  │
                        //  ├─┼─┼─┤
                        //  │  │E7│  │
                        //  └─┴─┴─┘
                        bool isE = true;

                        List<Process> srcAll = new List<Process>();
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 桂
                //************************************************************
                // 桂
                //************************************************************
                case KomaSyurui.KEI:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┐　┌─┐
                        //│  │　│  │
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│Ｊ│　│Ｉ│
                        //└─┘　└─┘
                        bool isI = true;
                        bool isJ = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isI = false;
                                isJ = false;
                                break;
                            case AgaruHiku.Agaru:
                                break;
                            case AgaruHiku.Hiku:
                                isI = false;
                                isJ = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isJ = false;
                                break;
                            case MigiHidari.Hidari:
                                isI = false;
                                break;
                            case MigiHidari.Sugu:
                                isI = false;
                                isJ = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isI) { srcAll.AddRange(KomanoKidou.SrcI_Kei(sengo, dst1)); }
                        if (isJ) { srcAll.AddRange(KomanoKidou.SrcJ_Kei(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion


                #region 銀
                //************************************************************
                // 銀
                //************************************************************
                case KomaSyurui.GIN:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│  │Ｂ│
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isB = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isB = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isD = false;
                                isE = false;
                                isF = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isB = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isB = false;
                                isD = false;
                                isF = false;
                                isH = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                // △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                case KomaSyurui.KIN:
                case KomaSyurui.TOKIN:
                case KomaSyurui.NARIKYO:
                case KomaSyurui.NARIKEI:
                case KomaSyurui.NARIGIN:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│  │Ａ│  │
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isA = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 王
                //************************************************************
                // 王
                //************************************************************
                case KomaSyurui.OH:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│Ａ│Ｂ│
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        List<Process> srcAll = new List<Process>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        // 王は１つです。
                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 竜
                //************************************************************
                // 竜
                //************************************************************
                case KomaSyurui.RYU:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｈ│A0│Ｂ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｆ│E0│Ｄ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isB = false;
                                isC = false;
                                isG = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Hisya(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Hisya(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Hisya(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 馬
                //************************************************************
                // 馬
                //************************************************************
                case KomaSyurui.UMA:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│Ａ│B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｇ│至│Ｃ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│Ｅ│D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isB = false;
                                isC = false;
                                isG = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                        }

                        List<Process> srcAll = new List<Process>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Kaku(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Kaku(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Kaku(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Kaku(sengo, dst1)); }

                        if (Translator_FugotextToProcess.Hit(sengo, srcSyurui, srcAll, kyokumen, out found))
                        {
                            srcOkiba = Okiba.SHOGIBAN;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region エラー
                //************************************************************
                // エラー
                //************************************************************
                default:
                    {
                    }
                    break;
                #endregion

            }

        gt_EndShogiban:

            if (null == found && utsu)
            {
                // 駒台の駒を(明示的に)打ちます。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                List<Button_Koma> btnKomas = kyokumen.Btns_ByOkibaSengoSyurui(srcOkiba, sengo, srcSyurui);

                if (0 < btnKomas.Count)
                {
                    switch (sengo)
                    {
                        case Sengo.GOTE:
                            srcOkiba = Okiba.GOTE_KOMADAI;
                            break;
                        case Sengo.SENTE:
                            srcOkiba = Okiba.SENTE_KOMADAI;
                            break;
                        default:
                            srcOkiba = Okiba.ERROR;
                            break;
                    }

                    found = btnKomas[0];
                    goto gt_EndSyurui;
                }
            }

        gt_EndSyurui:


            int srcSuji;
            int srcDan;
            if (null != found)
            {
                // 将棋盤の上に駒がありました。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                srcSuji = found.KomaProperty.Suji;
                srcDan = found.KomaProperty.Dan;
            }
            else
            {
                // （符号に書かれていませんが）「打」のとき。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                switch (sengo)
                {
                    case Sengo.GOTE:
                        srcOkiba = Okiba.GOTE_KOMADAI;
                        break;
                    case Sengo.SENTE:
                        srcOkiba = Okiba.SENTE_KOMADAI;
                        break;
                    default:
                        srcOkiba = Okiba.ERROR;
                        break;
                }

                srcSuji = Process.DA_SUJI;
                srcDan = Process.DA_DAN;
            }


            KomaSyurui syurui;
            if (NariFunari.Nari == nariFunari)
            {
                // 成ります
                syurui = Converter.SyuruiToNariCase(srcSyurui);
            }
            else
            {
                // そのままです。
                syurui = srcSyurui;
            }

            // 棋譜
            process = new Process(
                sengo,
                Okiba.SHOGIBAN, //符号は将棋盤の升目です。
                srcOkiba,
                suji,
                dan,
                srcSuji,
                srcDan,
                syurui,
                srcSyurui,
                KomaSyurui.UNKNOWN // 取った駒の種類は分からない
            );


        }


        /// <summary>
        /// 将棋盤上での検索
        /// </summary>
        /// <param name="srcAll">候補マス</param>
        /// <param name="komas"></param>
        /// <returns></returns>
        private static bool Hit(Sengo sengo, KomaSyurui syurui, List<Process> srcAll, Kyokumen kyokumen, out Button_Koma found)
        {
            bool hit = false;
            found = null;

            foreach (Process masu in srcAll)//筋・段。（先後、種類は入っていません）
            {
                foreach (Button_Koma btnKoma in kyokumen.Items)//全駒
                {
                    if (sengo == btnKoma.KomaProperty.Sengo
                        && Okiba.SHOGIBAN== btnKoma.KomaProperty.Okiba
                        && syurui == btnKoma.KomaProperty.Syurui
                        && masu.MatchMasu(btnKoma.KomaProperty)
                        )
                    {
                        // 候補マスにいた
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                        hit = true;
                        found = btnKoma;
                        break;
                    }
                }
            }

            return hit;
        }

    }


}
