﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;
using System.Text.RegularExpressions;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 「7g7f」といった記述を、棋譜データの持ち方に変換します。
    /// </summary>
    public abstract class Translator_SfenToProcess
    {

        /// <summary>
        /// テキスト形式の符号「7g7f 3c3d 6g6f…」の最初の要素を、切り取ってプロセスに変換します。
        /// 
        /// 再生、コマ送りで利用。
        /// </summary>
        /// <returns></returns>
        public static bool TranslateSfenToProcess(string text, out string restText, out Process process, Kifu baseKifu, Kyokumen kyokumen, Taikyoku taikyoku)
        {
            process = null;

            bool successful = false;

            restText = text;

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            Regex regex = new Regex(
                @"^\s*([123456789PLNSGKRB])([abcdefghi\*])([123456789])([abcdefghi])(\+)?",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    Translator_SfenToProcess.TranslateArgsToProcess(
                        m.Groups[1].Value,  //123456789 か、 PLNSGKRB
                        m.Groups[2].Value,  //abcdefghi か、 *
                        m.Groups[3].Value,  //123456789
                        m.Groups[4].Value,  //abcdefghi
                        m.Groups[5].Value,  //+
                        out process,
                        kyokumen,
                        taikyoku
                        );
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


        /// <summary>
        /// 符号１「7g7f」を元に、process を作ります。
        /// 
        /// ＜再生、コマ送りで呼び出されます＞
        /// </summary>
        /// <returns></returns>
        public static void TranslateArgsToProcess(
            string str1, //123456789 か、 PLNSGKRB
            string str2, //abcdefghi か、 *
            string str3, //123456789
            string str4, //abcdefghi
            string strNari, //+
            out Process process,
            Kyokumen kyokumen,
            Taikyoku taikyoku
            )
        {
            //------------------------------
            // 1
            //------------------------------
            KomaSyurui uttaSyurui; // 打った駒の種類
            Converter.SfenUttaSyurui(str1, out uttaSyurui);

            int srcSuji = Process.ERROR_SUJI;
            if (KomaSyurui.UNKNOWN == uttaSyurui)
            {
                if (!int.TryParse(str1, out srcSuji))
                {
                }
            }
            else
            {
            }

            //------------------------------
            // 2
            //------------------------------
            int srcDan = Process.ERROR_DAN;
            if ("*" == str2)
            {
                // 「打」でした。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            }
            else
            {
                srcDan = Converter.AlphabetToInt(str2);
            }

            //------------------------------
            // 3
            //------------------------------
            int suji;
            if(!int.TryParse(str3, out suji))
            {
            }

            //------------------------------
            // 4
            //------------------------------
            int dan;
            dan = Converter.AlphabetToInt(str4);

            

            Button_Koma btnKoma = kyokumen.Btn_AtMasuDokodemo(Okiba.SHOGIBAN, new Process(
                Sengo.NO_USE_ARG,
                Okiba.NO_USE,
                srcSuji,
                srcDan,
                KomaSyurui.UNKNOWN
                ));

            KomaSyurui syurui;
            KomaSyurui srcSyurui;
            Okiba srcOkiba;

            if (null != btnKoma)
            {
                // 盤上の駒を指した場合
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                syurui = btnKoma.KomaProperty.Syurui;
                srcSyurui = btnKoma.KomaProperty.Syurui; //駒は「元・種類」を記憶していませんので、「現・種類」を指定します。
                srcOkiba = Okiba.SHOGIBAN;
            }
            else
            {
                // 打った駒の場合
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                syurui = uttaSyurui;
                srcSyurui = uttaSyurui;
                switch (taikyoku.Sengo)
                {
                    case Sengo.GOTE:
                        srcOkiba = Okiba.GOTE_KOMADAI;
                        break;
                    case Sengo.SENTE:
                        srcOkiba = Okiba.SENTE_KOMADAI;
                        break;
                    default:
                        srcOkiba = Okiba.ERROR;
                        break;
                }
            }


            //------------------------------
            // 5
            //------------------------------
            if ("+" == strNari)
            {
                // 成りました
                syurui = Converter.SyuruiToNariCase(syurui);
            }

            //------------------------------
            // 結果
            //------------------------------
            // 棋譜
            process = new Process(
                taikyoku.Sengo,
                Okiba.SHOGIBAN, //符号は将棋盤の升目です。
                srcOkiba,
                suji,
                dan,
                srcSuji,
                srcDan,
                syurui,
                srcSyurui,
                KomaSyurui.UNKNOWN//取った駒は分からない
            );
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="srcAll">候補マス</param>
        /// <param name="komas"></param>
        /// <returns></returns>
        private static bool Hit(Sengo sengo, KomaSyurui syurui, List<Process> srcAll, Kyokumen kyokumen, out Button_Koma found)
        {
            bool hit = false;
            found = null;

            foreach (Process masu in srcAll)//筋・段。（先後、種類は入っていません）
            {
                foreach (Button_Koma btnKoma in kyokumen.Items)//全駒
                {
                    if (sengo == btnKoma.KomaProperty.Sengo
                        && syurui == btnKoma.KomaProperty.Syurui
                        && masu.MatchMasu(btnKoma.KomaProperty)
                        )
                    {
                        // 候補マスにいた
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                        hit = true;
                        found = btnKoma;
                        break;
                    }
                }
            }

            return hit;
        }

    }


}
