﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

namespace Xenon.KifuNarabe
{


    /// <summary>
    /// 自作ボタン
    /// </summary>
    public class Button2
    {

        #region プロパティー

        /// <summary>
        /// 表示テキスト
        /// </summary>
        public string Text
        {
            get;
            set;
        }

        /// <summary>
        /// 位置とサイズ
        /// </summary>
        public Rectangle Bounds
        {
            get;
            set;
        }

        /// <summary>
        /// 光
        /// </summary>
        public bool Light
        {
            get;
            set;
        }

        /// <summary>
        /// 動かしたい駒
        /// </summary>
        public bool Select
        {
            get;
            set;
        }

        /// <summary>
        /// 動かしたい駒を、最初に触ったときか
        /// </summary>
        public bool SelectFirstTouch
        {
            get;
            set;
        }

        public float FontSize
        {
            get;
            set;
        }

        #endregion


        public Button2()
        {
            this.Text = "ボタン";
            this.Bounds = new Rectangle(12,42,70,35);
            this.FontSize = 20.0f;
        }

        public Button2(string label, int x, int y)
            : this()
        {
            this.Text = label;
            this.Bounds = new Rectangle(x,y,this.Bounds.Width,this.Bounds.Height);
        }

        public Button2(string label, int x, int y, int width, int height, float fontSize)
            : this()
        {
            this.Text = label;
            this.Bounds = new Rectangle(x, y, width, height);
            this.FontSize = fontSize;
        }

        /// <summary>
        /// 描画
        /// </summary>
        /// <param name="g1"></param>
        public void Paint(Graphics g1)
        {
            //----------
            // 背景
            //----------
            if (this.Select)
            {
                g1.FillRectangle(Brushes.Green, this.Bounds);
            }
            else if (this.Light)
            {
            }


            //----------
            // 枠線
            //----------
            {
                Pen pen;
                if (this.Light)
                {
                    pen = Pens.Yellow;
                }
                else
                {
                    pen = Pens.Black;
                }

                g1.DrawRectangle(pen, this.Bounds);
            }

            //----------
            // 文字
            //----------
            g1.DrawString(this.Text, new Font(FontFamily.GenericSerif, this.FontSize), Brushes.Black, this.Bounds.Location);
        }

        /// <summary>
        /// マウスカーソルに重なっているか否か。
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool HitByMouse(int x, int y)
        {
            return this.Bounds.Contains(x, y);
        }

        /// <summary>
        /// マウスが重なった駒は、光フラグを立てます。
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void LightByMouse(int x, int y)
        {
            if (this.HitByMouse(x,y)) // マウスが重なっているなら
            {
                this.Light = true;
            }
            else // マウスが重なっていないなら
            {
                this.Light = false;
            }
        }

        /// <summary>
        /// 動かしたい駒
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void SelectByMouse(int x, int y)
        {
            // 既に選択されている駒には無効
            if (this.Select)
            {
                goto gt_EndMethod;
            }


            if (this.HitByMouse(x, y)) // マウスが重なっているなら
            {
                this.Select = true;
                this.SelectFirstTouch = true;
            }
            else
            {
                this.Select = false;
                this.SelectFirstTouch = false;
            }

        gt_EndMethod:
            ;
        }

        /// <summary>
        /// 動かしたい駒の解除
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool DeselectByMouse(int x, int y)
        {
            bool changed = false;

            if (this.HitByMouse(x, y)) // マウスが重なっているなら
            {
                if (this.SelectFirstTouch)
                {
                    // クリックのマウスアップ
                    this.SelectFirstTouch = false;
                }
                else
                {
                    // 選択解除のマウスアップ
                    this.Select = false;
                    changed = true;
                }
            }
            else
            {
                // 何もしない
            }

            return changed;
        }
    
    }


}
