﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xenon.KifuNarabe
{
    public abstract class Converter
    {

        private static string[] ARABIA_SUJI = new string[] { "１", "２", "３", "４", "５", "６", "７", "８", "９" };

        private static string[] KAN_SUJI = new string[] { "一", "二", "三", "四", "五", "六", "七", "八", "九" };


        /// <summary>
        /// a～i を、1～9 に変換します。
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static int AlphabetToInt(string alphabet)
        {
            int num;

            switch (alphabet)
            {
                case "a":
                    num = 1;
                    break;
                case "b":
                    num = 2;
                    break;
                case "c":
                    num = 3;
                    break;
                case "d":
                    num = 4;
                    break;
                case "e":
                    num = 5;
                    break;
                case "f":
                    num = 6;
                    break;
                case "g":
                    num = 7;
                    break;
                case "h":
                    num = 8;
                    break;
                case "i":
                    num = 9;
                    break;
                default:
                    num = -1;
                    break;
            }

            return num;
        }



        public static string IntToArabiaSuji(int num)
        {
            string numStr;

            if (1 <= num && num <= 9)
            {
                numStr = ARABIA_SUJI[num - 1];
            }
            else
            {
                numStr = "×";
            }

            return numStr;
        }

        public static string IntToKanSuji(int num)
        {
            string numStr;

            if (1 <= num && num <= 9)
            {
                numStr = KAN_SUJI[num - 1];
            }
            else
            {
                numStr = "×";
            }

            return numStr;
        }

        /// <summary>
        /// 1～9 を、a～i に変換します。
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static string IntToAlphabet(int suji)
        {
            string alphabet;

            switch (suji)
            {
                case 1:
                    alphabet = "a";
                    break;
                case 2:
                    alphabet = "b";
                    break;
                case 3:
                    alphabet = "c";
                    break;
                case 4:
                    alphabet = "d";
                    break;
                case 5:
                    alphabet = "e";
                    break;
                case 6:
                    alphabet = "f";
                    break;
                case 7:
                    alphabet = "g";
                    break;
                case 8:
                    alphabet = "h";
                    break;
                case 9:
                    alphabet = "i";
                    break;
                default:
                    alphabet = "×";
                    break;
            }

            return alphabet;
        }

        /// <summary>
        /// アラビア数字（全角半角）、漢数字を、int型に変換します。変換できなかった場合、0です。
        /// </summary>
        /// <param name="suji"></param>
        /// <returns></returns>
        public static int SujiToInt(string suji)
        {
            int result;

            switch (suji)
            {
                case "1":
                case "１":
                case "一":
                    result = 1;
                    break;

                case "2":
                case "２":
                case "二":
                    result = 2;
                    break;

                case "3":
                case "３":
                case "三":
                    result = 3;
                    break;

                case "4":
                case "４":
                case "四":
                    result = 4;
                    break;

                case "5":
                case "５":
                case "五":
                    result = 5;
                    break;

                case "6":
                case "６":
                case "六":
                    result = 6;
                    break;

                case "7":
                case "７":
                case "七":
                    result = 7;
                    break;

                case "8":
                case "８":
                case "八":
                    result = 8;
                    break;

                case "9":
                case "９":
                case "九":
                    result = 9;
                    break;

                default:
                    result = 0;
                    break;

            }

            return result;
        }


        /// <summary>
        /// 駒の表面の文字
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToLabel(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.FU:
                    koma = "歩";
                    break;

                case KomaSyurui.KYO:
                    koma = "香";
                    break;

                case KomaSyurui.KEI:
                    koma = "桂";
                    break;

                case KomaSyurui.GIN:
                    koma = "銀";
                    break;

                case KomaSyurui.KIN:
                    koma = "金";
                    break;

                case KomaSyurui.HISYA:
                    koma = "飛";
                    break;

                case KomaSyurui.KAKU:
                    koma = "角";
                    break;

                case KomaSyurui.OH:
                    koma = "王";
                    break;

                case KomaSyurui.TOKIN:
                    koma = "と";//と金
                    break;

                case KomaSyurui.NARIKYO:
                    koma = "杏";//成香
                    break;

                case KomaSyurui.NARIKEI:
                    koma = "圭";//成桂
                    break;

                case KomaSyurui.NARIGIN:
                    koma = "全";//成銀
                    break;

                case KomaSyurui.RYU:
                    koma = "竜";
                    break;

                case KomaSyurui.UMA:
                    koma = "馬";
                    break;

                case KomaSyurui.UNKNOWN:
                    koma = "Ｕ種";
                    break;

                default:
                    koma = "×";
                    break;
            }

            return koma;
        }

        /// <summary>
        /// 成り駒なら真。
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static bool IsSyuruiNari(KomaSyurui syurui)
        {
            bool isNari;

            switch (syurui)
            {
                case KomaSyurui.TOKIN:
                case KomaSyurui.NARIKYO:
                case KomaSyurui.NARIKEI:
                case KomaSyurui.NARIGIN:
                case KomaSyurui.RYU:
                case KomaSyurui.UMA:
                    isNari = true;
                    break;
                default:
                    isNari = false;
                    break;
            }

            return isNari;
        }

        /// <summary>
        /// 棋譜の符号用の駒の文字
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToFugo(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.FU:
                    koma = "歩";
                    break;

                case KomaSyurui.KYO:
                    koma = "香";
                    break;

                case KomaSyurui.KEI:
                    koma = "桂";
                    break;

                case KomaSyurui.GIN:
                    koma = "銀";
                    break;

                case KomaSyurui.KIN:
                    koma = "金";
                    break;

                case KomaSyurui.HISYA:
                    koma = "飛";
                    break;

                case KomaSyurui.KAKU:
                    koma = "角";
                    break;

                case KomaSyurui.OH:
                    koma = "王";
                    break;

                case KomaSyurui.TOKIN:
                    koma = "と";
                    break;

                case KomaSyurui.NARIKYO:
                    koma = "成香";
                    break;

                case KomaSyurui.NARIKEI:
                    koma = "成桂";
                    break;

                case KomaSyurui.NARIGIN:
                    koma = "成銀";
                    break;

                case KomaSyurui.RYU:
                    koma = "竜";
                    break;

                case KomaSyurui.UMA:
                    koma = "馬";
                    break;

                case KomaSyurui.MISETTEI_CUR:
                    koma = "MC";
                    break;

                case KomaSyurui.MISETTEI_PRI:
                    koma = "MP";
                    break;

                case KomaSyurui.KOMADAI_NI_NAKATTA:
                    koma = "KN";
                    break;

                case KomaSyurui.UNKNOWN:
                    koma = "Ｕ符";
                    break;

                default:
                    koma = "×";
                    break;
            }

            return koma;
        }

        /// <summary>
        /// 棋譜の符号用の駒の文字。先手玉→K、後手玉→k など。
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToSfen(Sengo sengo, KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.FU:
                    koma = "P";
                    break;

                case KomaSyurui.KYO:
                    koma = "L";
                    break;

                case KomaSyurui.KEI:
                    koma = "N";
                    break;

                case KomaSyurui.GIN:
                    koma = "S";
                    break;

                case KomaSyurui.KIN:
                    koma = "G";
                    break;

                case KomaSyurui.HISYA:
                    koma = "R";
                    break;

                case KomaSyurui.KAKU:
                    koma = "B";
                    break;

                case KomaSyurui.OH:
                    koma = "K";
                    break;

                case KomaSyurui.TOKIN:
                    koma = "+P";
                    break;

                case KomaSyurui.NARIKYO:
                    koma = "+L";
                    break;

                case KomaSyurui.NARIKEI:
                    koma = "+N";
                    break;

                case KomaSyurui.NARIGIN:
                    koma = "+S";
                    break;

                case KomaSyurui.RYU:
                    koma = "+R";
                    break;

                case KomaSyurui.UMA:
                    koma = "+B";
                    break;

                case KomaSyurui.MISETTEI_CUR:
                    koma = "MC";
                    break;

                case KomaSyurui.MISETTEI_PRI:
                    koma = "MP";
                    break;

                case KomaSyurui.KOMADAI_NI_NAKATTA:
                    koma = "KN";
                    break;

                case KomaSyurui.UNKNOWN:
                    koma = "ＵSfen";
                    break;

                default:
                    koma = "×";
                    break;
            }

            if (Sengo.GOTE == sengo)
            {
                koma = koma.ToLower();
            }

            return koma;
        }

        /// <summary>
        /// 棋譜の符号用の駒の文字。打用。
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToSfenDa(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.FU:
                case KomaSyurui.TOKIN:
                    koma = "P";
                    break;

                case KomaSyurui.KYO:
                case KomaSyurui.NARIKYO:
                    koma = "L";
                    break;

                case KomaSyurui.KEI:
                case KomaSyurui.NARIKEI:
                    koma = "N";
                    break;

                case KomaSyurui.GIN:
                case KomaSyurui.NARIGIN:
                    koma = "S";
                    break;

                case KomaSyurui.KIN:
                    koma = "G";
                    break;

                case KomaSyurui.HISYA:
                case KomaSyurui.RYU:
                    koma = "R";
                    break;

                case KomaSyurui.KAKU:
                case KomaSyurui.UMA:
                    koma = "B";
                    break;

                case KomaSyurui.OH:
                    koma = "K";
                    break;

                case KomaSyurui.MISETTEI_CUR:
                    koma = "＜打MC＞";
                    break;

                case KomaSyurui.MISETTEI_PRI:
                    koma = "＜打MP＞";
                    break;

                case KomaSyurui.KOMADAI_NI_NAKATTA:
                    koma = "＜打KN＞";
                    break;

                case KomaSyurui.UNKNOWN:
                    koma = "＜打Ｕ＞";
                    break;

                default:
                    koma = "＜打×＞";
                    break;
            }

            return koma;
        }

        /// <summary>
        /// 駒の種類。
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static void SfenSyokihaichiToSyurui(string sfen, out Sengo sengo, out KomaSyurui syurui)
        {
            switch (sfen)
            {
                case "P":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.FU;
                    break;

                case "p":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.FU;
                    break;

                case "L":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.KYO;
                    break;

                case "l":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.KYO;
                    break;

                case "N":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.KEI;
                    break;

                case "n":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.KEI;
                    break;

                case "S":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.GIN;
                    break;

                case "s":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.GIN;
                    break;

                case "G":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.KIN;
                    break;

                case "g":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.KIN;
                    break;

                case "R":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.HISYA;
                    break;

                case "r":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.HISYA;
                    break;

                case "B":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.KAKU;
                    break;

                case "b":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.KAKU;
                    break;

                case "K":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.OH;
                    break;

                case "k":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.OH;
                    break;

                case "+P":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.TOKIN;
                    break;

                case "+p":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.TOKIN;
                    break;

                case "+L":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.NARIKYO;
                    break;

                case "+l":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.NARIKYO;
                    break;

                case "+N":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.NARIKEI;
                    break;

                case "+n":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.NARIKEI;
                    break;

                case "+S":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.NARIGIN;
                    break;

                case "+s":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.NARIGIN;
                    break;

                case "+R":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.HISYA;
                    break;

                case "+r":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.HISYA;
                    break;

                case "+B":
                    sengo = Sengo.SENTE;
                    syurui = KomaSyurui.KAKU;
                    break;

                case "+b":
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.KAKU;
                    break;

                default:
                    sengo = Sengo.GOTE;
                    syurui = KomaSyurui.UNKNOWN;
                    break;
            }
        }


        /// <summary>
        /// 打った駒の種類。
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static void SfenUttaSyurui(string sfen, out KomaSyurui syurui)
        {
            switch (sfen)
            {
                case "P":
                    syurui = KomaSyurui.FU;
                    break;

                case "L":
                    syurui = KomaSyurui.KYO;
                    break;

                case "N":
                    syurui = KomaSyurui.KEI;
                    break;

                case "S":
                    syurui = KomaSyurui.GIN;
                    break;

                case "G":
                    syurui = KomaSyurui.KIN;
                    break;

                case "R":
                    syurui = KomaSyurui.HISYA;
                    break;

                case "B":
                    syurui = KomaSyurui.KAKU;
                    break;

                case "K":
                    syurui = KomaSyurui.OH;
                    break;

                case "+P":
                    syurui = KomaSyurui.TOKIN;
                    break;

                case "+L":
                    syurui = KomaSyurui.NARIKYO;
                    break;

                case "+N":
                    syurui = KomaSyurui.NARIKEI;
                    break;

                case "+S":
                    syurui = KomaSyurui.NARIGIN;
                    break;

                case "+R":
                    syurui = KomaSyurui.HISYA;
                    break;

                case "+B":
                    syurui = KomaSyurui.KAKU;
                    break;

                default:
                    syurui = KomaSyurui.UNKNOWN;
                    break;
            }
        }


        /// <summary>
        /// 駒の文字を、列挙型へ変換。
        /// </summary>
        /// <param name="moji"></param>
        /// <returns></returns>
        public static KomaSyurui MojiToSyurui(string moji)
        {
            KomaSyurui syurui;

            switch (moji)
            {
                case "歩":
                    syurui = KomaSyurui.FU;
                    break;

                case "香":
                    syurui = KomaSyurui.KYO;
                    break;

                case "桂":
                    syurui = KomaSyurui.KEI;
                    break;

                case "銀":
                    syurui = KomaSyurui.GIN;
                    break;

                case "金":
                    syurui = KomaSyurui.KIN;
                    break;

                case "飛":
                    syurui = KomaSyurui.HISYA;
                    break;

                case "角":
                    syurui = KomaSyurui.KAKU;
                    break;

                case "王"://thru
                case "玉":
                    syurui = KomaSyurui.OH;
                    break;

                case "と":
                    syurui = KomaSyurui.TOKIN;
                    break;

                case "成香":
                    syurui = KomaSyurui.NARIKYO;
                    break;

                case "成桂":
                    syurui = KomaSyurui.NARIKEI;
                    break;

                case "成銀":
                    syurui = KomaSyurui.NARIGIN;
                    break;

                case "竜"://thru
                case "龍":
                    syurui = KomaSyurui.RYU;
                    break;

                case "馬":
                    syurui = KomaSyurui.UMA;
                    break;

                default:
                    syurui = KomaSyurui.UNKNOWN;
                    break;
            }

            return syurui;
        }

        /// <summary>
        /// 種類を見て、成／不成を判定
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static bool SyuruiToNari(KomaSyurui syurui)
        {
            bool nari;

            switch (syurui)
            {
                //----------
                // 成
                //----------
                case KomaSyurui.TOKIN: //と金
                case KomaSyurui.NARIKYO: //成香
                case KomaSyurui.NARIKEI: //成桂
                case KomaSyurui.NARIGIN: //成銀
                case KomaSyurui.RYU: //竜
                case KomaSyurui.UMA: //馬
                    nari = true;
                    break;

                //----------
                // 不成
                //----------
                case KomaSyurui.FU: //歩
                case KomaSyurui.KYO: //香
                case KomaSyurui.KEI: //桂
                case KomaSyurui.GIN: //銀
                case KomaSyurui.KIN: //金
                case KomaSyurui.HISYA: //飛
                case KomaSyurui.KAKU: //角
                case KomaSyurui.OH: //王
                default:
                    nari = false;
                    break;
            }

            return nari;
        }

        /// <summary>
        /// 種類を見て、不成の種類を判定
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static KomaSyurui SyuruiToFunariCase(KomaSyurui syurui)
        {
            KomaSyurui funari;

            switch (syurui)
            {
                //----------
                // 成 → 不成
                //----------
                case KomaSyurui.TOKIN: //と金
                    funari = KomaSyurui.FU;
                    break;

                case KomaSyurui.NARIKYO: //成香
                    funari = KomaSyurui.KYO;
                    break;

                case KomaSyurui.NARIKEI: //成桂
                    funari = KomaSyurui.KEI;
                    break;

                case KomaSyurui.NARIGIN: //成銀
                    funari = KomaSyurui.GIN;
                    break;

                case KomaSyurui.RYU: //竜
                    funari = KomaSyurui.HISYA;
                    break;

                case KomaSyurui.UMA: //馬
                    funari = KomaSyurui.KAKU;
                    break;

                //----------
                // 不成
                //----------
                case KomaSyurui.FU: //歩
                case KomaSyurui.KYO: //香
                case KomaSyurui.KEI: //桂
                case KomaSyurui.GIN: //銀
                case KomaSyurui.KIN: //金
                case KomaSyurui.HISYA: //飛
                case KomaSyurui.KAKU: //角
                case KomaSyurui.OH: //王
                default:
                    funari = syurui;
                    break;
            }

            return funari;
        }

        /// <summary>
        /// 種類を見て、成りの種類を判定
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static KomaSyurui SyuruiToNariCase(KomaSyurui syurui)
        {
            KomaSyurui nari;

            switch (syurui)
            {
                //----------
                // 
                //----------

                //----------
                // 不成→成
                //----------
                case KomaSyurui.FU: //歩
                    nari = KomaSyurui.TOKIN;
                    break;

                case KomaSyurui.KYO: //香
                    nari = KomaSyurui.NARIKYO;
                    break;

                case KomaSyurui.KEI: //桂
                    nari = KomaSyurui.NARIKEI;
                    break;

                case KomaSyurui.GIN: //銀
                    nari = KomaSyurui.NARIGIN;
                    break;

                case KomaSyurui.HISYA: //飛
                    nari = KomaSyurui.RYU;
                    break;

                case KomaSyurui.KAKU: //角
                    nari = KomaSyurui.UMA;
                    break;

                //----------
                // 成　or　成のない駒
                //----------
                case KomaSyurui.TOKIN: //と金
                case KomaSyurui.NARIKYO: //成香
                case KomaSyurui.NARIKEI: //成桂
                case KomaSyurui.NARIGIN: //成銀
                case KomaSyurui.RYU: //竜
                case KomaSyurui.UMA: //馬

                case KomaSyurui.KIN: //金
                case KomaSyurui.OH: //王
                default:
                    nari = syurui;
                    break;
            }

            return nari;
        }


        public static string BoolToDa(DaHyoji da)
        {
            string daStr = "";

            if (DaHyoji.Visible == da)
            {
                daStr = "打";
            }

            return daStr;
        }

        public static DaHyoji StrToDaHyoji(string daStr)
        {
            DaHyoji daHyoji = DaHyoji.Invisible;

            if (daStr == "打")
            {
                daHyoji = DaHyoji.Visible;
            }

            return daHyoji;
        }

        public static string BoolToDou(bool dou)
        {
            string douStr = "";

            if (dou)
            {
                douStr = "同";
            }

            return douStr;
        }

        public static string NariToStr(NariFunari nari)
        {
            string nariStr = "";

            switch(nari)
            {
                case NariFunari.Nari:
                    nariStr = "成";
                    break;
                case NariFunari.Funari:
                    nariStr = "不成";
                    break;
                default:
                    break;
            }

            return nariStr;
        }

        public static NariFunari NariToBool(string nariStr)
        {
            NariFunari nari;

            if ("成" == nariStr)
            {
                nari = NariFunari.Nari;
            }
            else if ("不成" == nariStr)
            {
                nari = NariFunari.Funari;
            }
            else
            {
                nari = NariFunari.None;
            }

            return nari;
        }

        public static Sengo SengoToEnum(string sengoStr)
        {
            Sengo sengo;

            switch (sengoStr)
            {
                case "△":
                    sengo = Sengo.GOTE;
                    break;

                case "▲":
                default:
                    sengo = Sengo.SENTE;
                    break;
            }

            return sengo;
        }

        public static string SengoToStr(Sengo sengo)
        {
            string sengoStr;

            switch (sengo)
            {
                case Sengo.GOTE:
                    sengoStr = "△";
                    break;

                case Sengo.SENTE:
                default:
                    sengoStr = "▲";
                    break;
            }

            return sengoStr;
        }

        public static string SengoToKanji(Sengo sengo)
        {
            string sengoStr;

            switch (sengo)
            {
                case Sengo.SENTE:
                    sengoStr = "先手";
                    break;
                case Sengo.GOTE:
                    sengoStr = "後手";
                    break;
                default:
                    sengoStr = "×";
                    break;
            }

            return sengoStr;
        }

        public static Okiba SengoToKomadaiOkiba(Sengo sengo)
        {
            Okiba okiba;

            switch (sengo)
            {
                case Sengo.GOTE:
                    okiba = Okiba.GOTE_KOMADAI;
                    break;

                case Sengo.SENTE://thru
                default:
                    okiba = Okiba.SENTE_KOMADAI;
                    break;
            }

            return okiba;
        }

        public static string MigiHidariToStr(MigiHidari migiHidari)
        {
            string str;

            switch (migiHidari)
            {
                case MigiHidari.Migi:
                    str = "右";
                    break;

                case MigiHidari.Hidari:
                    str = "左";
                    break;

                case MigiHidari.Sugu:
                    str = "直";
                    break;

                default:
                    str = "";
                    break;
            }

            return str;
        }

        /// <summary>
        /// 寄、右、左、直、なし
        /// </summary>
        /// <param name="migiHidariStr"></param>
        /// <returns></returns>
        public static MigiHidari StrToMigiHidari(string migiHidariStr)
        {
            MigiHidari migiHidari;

            switch (migiHidariStr)
            {
                case "右":
                    migiHidari = MigiHidari.Migi;
                    break;

                case "左":
                    migiHidari = MigiHidari.Hidari;
                    break;

                case "直":
                    migiHidari = MigiHidari.Sugu;
                    break;

                default:
                    migiHidari = MigiHidari.None;
                    break;
            }

            return migiHidari;
        }

        public static string AgaruHikuToStr(AgaruHiku agaruHiku)
        {
            string str;

            switch (agaruHiku)
            {
                case AgaruHiku.Yoru:
                    str = "寄";
                    break;

                case AgaruHiku.Hiku:
                    str = "引";
                    break;

                case AgaruHiku.Agaru:
                    str = "上";
                    break;

                default:
                    str = "";
                    break;
            }

            return str;
        }

        public static AgaruHiku StrToAgaruHiku(string agaruHikuStr)
        {
            AgaruHiku agaruHiku;

            switch (agaruHikuStr)
            {
                case "寄":
                    agaruHiku = AgaruHiku.Yoru;
                    break;

                case "引":
                    agaruHiku = AgaruHiku.Hiku;
                    break;

                case "上":
                    agaruHiku = AgaruHiku.Agaru;
                    break;

                default:
                    agaruHiku = AgaruHiku.None;
                    break;
            }

            return agaruHiku;
        }


        /// <summary>
        /// 先後の交代
        /// </summary>
        /// <param name="sengo">先後</param>
        /// <returns>ひっくりかえった先後</returns>
        public static Sengo AlternateSengo(Sengo sengo)
        {
            Sengo result;

            switch (sengo)
            {
                case Sengo.SENTE:
                    result = Sengo.GOTE;
                    break;

                case Sengo.GOTE:
                    result = Sengo.SENTE;
                    break;

                default:
                    result = sengo;
                    break;
            }

            return result;
        }

    }
}
